"""
Bounds class.

A bound is composed of :
- Its minimal x-value (minx)
- Its minimal y-value (miny)
- Its maximal x-value (maxx)
- Its maximal y-value (maxy)
- A srs (optional)
"""

import json


def removeBuffer(bounds):
    crop = '{"type": "filters.crop"}'
    parsed = json.loads(crop)
    parsed['bounds'] = str(bounds)
    return parsed


class Bounds:
    def __init__(self, minx, miny, maxx, maxy, srs=None):
        self.minx = minx
        self.miny = miny
        self.maxx = maxx
        self.maxy = maxy
        self.srs = srs
        self.distX = self.maxx - self.minx
        self.distY = self.maxy - self.miny

    def getDistX(self):
        return self.distX

    def getDistY(self):
        return self.distY

    def buffer(self, buffer):
        if buffer < 0:
            minx = self.minx + buffer
            miny = self.miny + buffer
            maxx = self.maxx - buffer
            maxy = self.maxy - buffer
        else:
            minx = self.minx - buffer
            miny = self.miny - buffer
            maxx = self.maxx + buffer
            maxy = self.maxy + buffer

        assign = '{"type": "filters.assign"}'
        parsed = json.loads(assign)
        parsed['value'] = f'ClassFlags = 2 WHERE X > {self.maxx} || X < {self.minx} || Y > {self.maxy} || Y < {self.miny}'
        return self, Bounds(minx, miny, maxx, maxy, self.srs), parsed

    def __str__(self):
        return f"([{self.minx:.2f},{self.maxx:.2f}],[{self.miny:.2f},{self.maxy:.2f}])"



"""
Do file.

Responsible for all the executions, serializations or creations of object we need to process the pipelines.
"""

import dask
from dask.distributed import Lock
from . import tile
from . import cloud
from . import bounds
import pickle
import os


@dask.delayed
def process(pipeline, temp_dir=None):
    """Process pipeline and delete the associate temp file if it's not a dry run"""
    if temp_dir:
        with Lock(str(pipeline[1])):
            # Get the temp file associated with the pipeline
            temp_file = temp_dir + '/' + str(pipeline[1]) + '.pickle'
        # Execute the pipeline
        pipeline[0].execute()
        try:
            # Remove the temp file
            os.remove(temp_file)
        except FileNotFoundError:
            pass
    # Don't need to get and suppress the temp file if it's a dry run
    else:
        pipeline[0].execute()


def process_serialized_pipelines(temp_dir, iterator):
    """Create the array of delayed tasks for pipelines in the temp directory (if there is some)"""
    delayedPipelines = []
    # yield loop syntax
    while True:
        try:
            p = next(iterator)
        except StopIteration:
            break

        # Add the delayed task to the array
        delayedPipelines.append(dask.delayed(process)(p, temp_dir))

    return delayedPipelines


def process_pipelines(output_dir, json_pipeline, iterator, temp_dir=None, dry_run=False, is_single=False):
    delayedPipelines = []
    while True:
        try:
            # If it's a cloud, 'next(iterator)' is a tile. Else, 'next(iterator)' is a filepath so a tile must be
            # created
            t = next(iterator) if is_single else tile.Tile(next(iterator), output_dir, json_pipeline)
            p = t.pipeline(is_single)
            # If it's not a dry run, the pipeline must be serialized
            if not dry_run:
                serializePipeline(p, temp_dir)
                delayedPipelines.append(dask.delayed(process)(p, temp_dir))
            # If it's a dry run, the pipeline is not serialized
            else:
                delayedPipelines.append(dask.delayed(process)(p))
        except StopIteration:
            break

    return delayedPipelines


def splitCloud(filepath, output_dir, json_pipeline, tile_bounds, nTiles=None, buffer=None, remove_buffer=False, bounding_box=None):
    """Split the cloud in many tiles"""
    bds = bounds.Bounds(bounding_box[0], bounding_box[1], bounding_box[2], bounding_box[3]) if bounding_box else None
    # Create a cloud object
    c = cloud.Cloud(filepath, bounds=bds)
    # Create a tile the size of the cloud
    t = tile.Tile(filepath=c.filepath, output_dir=output_dir, json_pipeline=json_pipeline, bounds=c.bounds, buffer=buffer, remove_buffer=remove_buffer, cloud_object=c)
    # Split the tile in small parts of given sizes
    return t.split(tile_bounds[0], tile_bounds[1], nTiles)


def serializePipeline(pipeline, temp_dir):
    """Serialize the pipelines"""
    # Create the temp file path
    temp_file = temp_dir + '/' + str(pipeline[1]) + '.pickle'
    with open(temp_file, 'wb') as outfile:
        # Serialize the pipeline
        pickle.dump(pipeline, outfile)



import json
import logging
import os
import os.path
import click
import pdal
from dask import config as cfg
from dask.distributed import LocalCluster, Client, progress
from distributed.diagnostics import MemorySampler
from os import listdir
from . import do
from . import file_manager
from . import cloud
from . import tile
from matplotlib import pyplot as plt
import sys
import ntpath
from os.path import join
from math import ceil


def query_yes_no(question, default='no'):
    valid = {'yes': True, 'y': True, 'ye': True, 'no': False, 'n': False}
    if default is None:
        prompt = ' [y/n] '
    elif default == 'yes':
        prompt = ' [Y/n] '
    elif default == 'no':
        prompt = ' [y/N] '
    else:
        raise ValueError(f'Invalid default answer: {default}')

    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write('Please respond with "yes" or "no" (or "y" or "n").\n')


def config_dask(n_workers, threads_per_worker, timeout):
    """Make some configuration to avoid workers errors due to heartbeat or timeout problems. Set the number of cores
    to process the pipelines """
    if not timeout:
        timeout = input('After how long of inactivity do you want to kill your worker (timeout)\n')

    cfg.set({'interface': 'lo'})
    cfg.set({'distributed.scheduler.worker-ttl': None})
    cfg.set({'distributed.comm.timeouts.connect': timeout})
    cluster = LocalCluster(n_workers=n_workers, threads_per_worker=threads_per_worker, silence_logs=logging.ERROR)
    client = Client(cluster)
    return client


def process_pipelines(
        config,
        input_type,
        timeout=None,
        n_workers=3,
        threads_per_worker=1,
        dry_run=None,
        diagnostic=False,
        tile_size=(256, 256),
        buffer=None,
        remove_buffer=None,
        bounding_box=None,
        merge_tiles=False,
        remove_tiles=False
):
    # Assertions
    assert type(config) is str
    assert input_type == "single" or input_type == "dir"
    if timeout:
        assert type(timeout) is int
    assert type(n_workers) is int
    assert type(threads_per_worker) is int
    if dry_run:
        assert type(dry_run) is int
    assert type(diagnostic) is bool
    assert type(tile_size) is tuple and len(tile_size) == 2
    if buffer:
        assert type(buffer) is int
    if remove_buffer:
        assert type(remove_buffer) is bool
    if bounding_box:
        assert type(bounding_box) is tuple
        assert len(bounding_box) == 4
    assert type(merge_tiles) is bool
    assert type(remove_tiles) is bool

    with open(config, 'r') as c:
        config_file = json.load(c)
        input_dir = config_file.get('input')
        output = config_file.get('output')
        temp = config_file.get('temp')
        pipeline = config_file.get('pipeline')

    if n_workers >= os.cpu_count():
        answer = query_yes_no(
            f'\nWARNING - You choose to launch {n_workers} workers but your machine has only {os.cpu_count()}'
            f' CPUs, please reduce the number of workers.\nDo you want to continue ?'
        )
        if not answer:
            return

    if input_type == 'single':
        if tile_size == (256, 256):
            answer = query_yes_no(
                f'WARNING - You are using the default value of the tile_size option (256 by 256 meters). Please '
                f'check if your points cloud\'s dimensions are greater than this value.\nDo you want to continue ? '
            )
            if not answer:
                return

        infos = cloud.compute_quickinfo(input_dir)
        bounds = infos['summary']['bounds']
        distX, distY = (
            int(bounds['maxx'] - bounds['minx']),
            int(bounds['maxy'] - bounds['miny'])
        )

        nTilesX = ceil(distX / tile_size[0])
        nTilesY = ceil(distY / tile_size[0])

        if nTilesX * nTilesY > n_workers:
            answer = query_yes_no(
                f'WARNING - With this size of tiles and this number of workers, each worker will have more than one task'
                f' and it can blow up the distributed memory. Please choose a larger size for your tiles or increase '
                f'your number of workers.\nDo you want to continue ?'
            )
            if not answer:
                return

    if not os.path.exists(temp):
        os.mkdir(temp)

    if not os.path.exists(output):
        os.mkdir(output)

    # If there is some temp file in the temp directory, these are processed
    if len(listdir(temp)) != 0:
        click.echo(
            f'Something went wrong during previous execution, there is some temp files in your temp '
            f'directory.\nBeginning of the execution\n')
        # Get all the deserialized pipelines
        pipeline_iterator = file_manager.getSerializedPipelines(temp_directory=temp)
        # Process pipelines
        delayed = do.process_serialized_pipelines(temp_dir=temp, iterator=pipeline_iterator)
    else:
        click.echo('Beginning of the execution\n')
        # If the user don't specify the dry_run option
        if not dry_run:
            # If the user wants to process a single file, it is split. Else, get all the files of the input directory
            iterator = do.splitCloud(filepath=input_dir,
                                     output_dir=output,
                                     json_pipeline=pipeline,
                                     tile_bounds=tile_size,
                                     buffer=buffer,
                                     remove_buffer=remove_buffer,
                                     bounding_box=bounding_box) if input_type == 'single' \
                else file_manager.getFiles(input_directory=input_dir)
            # Process pipelines
            delayed = do.process_pipelines(output_dir=output, json_pipeline=pipeline, temp_dir=temp, iterator=iterator,
                                           is_single=(input_type == 'single'))
        else:
            # If the user wants to process a single file, it is split and get the number of tiles given by the user.
            # Else, get the number of files we want to do the test execution (not serialized)
            iterator = do.splitCloud(filepath=input_dir,
                                     output_dir=output,
                                     json_pipeline=pipeline,
                                     tile_bounds=tile_size,
                                     nTiles=dry_run,
                                     buffer=buffer,
                                     remove_buffer=remove_buffer,
                                     bounding_box=bounding_box) if input_type == 'single' \
                else file_manager.getFiles(input_directory=input_dir, nFiles=dry_run)
            # Process pipelines
            delayed = do.process_pipelines(output_dir=output, json_pipeline=pipeline, iterator=iterator,
                                           dry_run=dry_run, is_single=(input_type == 'single'))

    client = config_dask(n_workers=n_workers, threads_per_worker=threads_per_worker, timeout=timeout)

    click.echo('Parallelization started.\n')

    if diagnostic:
        ms = MemorySampler()
        with ms.sample(label='execution', client=client):
            delayed = client.compute(delayed)
            progress(delayed)
        ms.plot()
    else:
        delayed = client.compute(delayed)
        progress(delayed)

    del delayed

    file_manager.getEmptyWeight(output_directory=output)

    if merge_tiles and len(listdir(output)) > 1:
        input_filename = ntpath.basename(input_dir).split('.')[0]
        writers = tile.get_writers(tile.load_pipeline(pipeline))
        merge = cloud.merge(output, input_filename, writers)
        if merge is not None:
            merge_ppln = pdal.Pipeline(merge)
            merge_ppln.execute()

        if remove_tiles:
            for f in os.listdir(output):
                if f.split('.')[0] != input_filename:
                    os.remove(join(output, f))

    plt.savefig(output + '/memory-usage.png') if diagnostic else None



"""
Main file.

Contains the process-pipelines function you call in command line
"""
import sys
import click
from pdal_parallelizer import process_pipelines as process


@click.group()
@click.version_option('2.0.3')
def main():
    """A simple parallelization tool for 3d point clouds treatment"""
    pass


@main.command()
@click.option('-c', '--config', required=True, type=click.Path(exists=True))
@click.option('-nw', '--n_workers', required=False, type=int, default=3)
@click.option('-tpw', '--threads_per_worker', required=False, type=int, default=1)
@click.option('-dr', '--dry_run', required=False, type=int)
@click.option('-d', '--diagnostic', is_flag=True, required=False)
@click.option('-it', '--input_type', required=True, type=click.Choice(['single', 'dir']))
@click.option('-ts', '--tile_size', required=False, nargs=2, type=int, default=(256, 256))
@click.option('-b', '--buffer', required=False, type=int)
@click.option('-rb', '--remove_buffer', is_flag=True, required=False)
@click.option('-bb', '--bounding_box', required=False, nargs=4, type=float)
@click.option('-mt', '--merge_tiles', required=False, is_flag=True)
@click.option('-rt', '--remove_tiles', required=False, is_flag=True)
def process_pipelines(**kwargs):
    # Get all the options
    config = kwargs.get('config')
    n_workers = kwargs.get('n_workers')
    threads_per_worker = kwargs.get('threads_per_worker')
    dry_run = kwargs.get('dry_run')
    diagnostic = kwargs.get('diagnostic')
    input_type = kwargs.get('input_type')
    tile_size = kwargs.get('tile_size')
    buffer = kwargs.get('buffer')
    remove_buffer = kwargs.get('remove_buffer')
    bounding_box = kwargs.get('bounding_box')
    merge_tiles = kwargs.get('merge_tiles')
    remove_tiles = kwargs.get('remove_tiles')

    process(
        config=config,
        input_type=input_type,
        n_workers=n_workers,
        threads_per_worker=threads_per_worker,
        dry_run=dry_run,
        diagnostic=diagnostic,
        tile_size=tile_size,
        buffer=buffer,
        remove_buffer=remove_buffer,
        bounding_box=bounding_box,
        merge_tiles=merge_tiles,
        remove_tiles=remove_tiles
    )


if __name__ == "__main__":
    args = sys.argv
    if "--help" in args or len(args) == 1:
        print("pdal_parallelizer")
    main()



"""
Cloud class.

A Cloud is composed of :
- Its path (filepath)
- Its limits (bounds)
"""

import subprocess
import json
from .bounds import Bounds
from os import listdir


def crop(bounds):
    crop = '{"type": "filters.crop"}'
    parsed = json.loads(crop)
    parsed['bounds'] = str(bounds)
    return parsed


def merge(output_dir, filename, writers):
    outputs = ""
    # Default values according to the pdal writers.las documentation
    compression = 'none'
    minor_version = 2
    dataformat_id = 3

    for f in listdir(output_dir):
        if f.split('.').count('png') <= 0:
            outputs += '"' + output_dir + '/' + f + '",'

    if outputs != "":
        extension = listdir(output_dir)[0].split('.')[1]
        if extension == 'laz':
            writers_extension = 'las'
            compression = 'laszip'
        else:
            writers_extension = extension

        try:
            minor_version = writers[0]['minor_version']
            dataformat_id = writers[0]['dataformat_id']
        except KeyError:
            pass

        merge = '[' + outputs + '{"type": "writers.' + writers_extension + '", "filename":"' + output_dir + '/' + \
                filename + '.' + extension + '","extra_dims": "all", "compression": "' + compression + '", ' + \
                '"minor_version": ' + str(minor_version) + ', "dataformat_id": ' + str(dataformat_id) + '}]'

        return merge


def addClassFlags():
    ferry = '{"type": "filters.ferry", "dimensions": "=>ClassFlags"}'
    return json.loads(ferry)


def compute_quickinfo(filepath):
    """Returns some information about the cloud."""
    # Get the cloud information
    pdal_info = subprocess.run(['pdal', 'info', filepath, '--summary'],
                               stderr=subprocess.PIPE,
                               stdout=subprocess.PIPE)
    info = json.loads(pdal_info.stdout.decode())

    return info


class Cloud:
    def __init__(self, filepath, bounds=None):
        self.filepath = filepath
        self.info = compute_quickinfo(self.filepath)
        self.classFlags = self.hasClassFlags()

        # Get the cloud information to set its bounds
        if bounds:
            minx, miny, maxx, maxy = bounds.minx, bounds.miny, bounds.maxx, bounds.maxy
        else:
            bounds_dict = self.info['summary']['bounds']
            minx, miny, = (
                bounds_dict['minx'],
                bounds_dict['miny']
            )

            maxx, maxy = (
                bounds_dict['maxx'],
                bounds_dict['maxy']
            )

        # Create bounds for the cloud
        self.bounds = Bounds(minx, miny, maxx, maxy)

    # Get the number of points in the cloud
    def getCount(self):
        return self.info['summary']['num_points']

    def hasClassFlags(self):
        """Check if the cloud has the ClassFlags dimension"""
        info = compute_quickinfo(self.filepath)
        dimensions = info['summary']['dimensions']
        return 'ClassFlags' in dimensions

    def __str__(self):
        return f'Cloud - {self.bounds}'



"""
Tile class.

A tile is composed of :
- Its path (filepath)
- A directory to store the results files (output_dir)
- A pdal pipeline (json_pipeline)
- A name (optional) (name)
- Its limits (optional) (bounds)
"""

import sys
import pdal
import json
import os
from . import cloud
from . import bounds


def load_pipeline(pipeline):
    with open(pipeline, 'r') as ppln:
        p = json.load(ppln)
    return p


def get_readers(pipeline):
    return list(filter(lambda x: x['type'].startswith('readers'), pipeline))


def get_writers(pipeline):
    return list(filter(lambda x: x['type'].startswith('writers'), pipeline))


class Tile:
    def __init__(self, filepath, output_dir, json_pipeline, name=None, bounds=None, buffer=None, remove_buffer=False, cloud_object=None):
        self.filepath = filepath
        self.output_dir = output_dir
        self.json_pipeline = json_pipeline

        if name:
            self.name = name
        else:
            self.name = os.path.basename(self.filepath).split('.')[0]

        self.buffer = buffer
        self.remove_buffer = remove_buffer

        if self.buffer:
            self.bounds_without_buffer, self.bounds, self.assign = bounds.buffer(self.buffer)
        else:
            self.bounds = bounds

        self.cloud = cloud_object

    def getName(self):
        return self.name

    def pipeline(self, single_file):
        """Assign a pipeline to the tile"""
        output_dir = self.output_dir

        p = load_pipeline(self.json_pipeline)

        # Create the name of the temp file associated to the pipeline
        temp_name = 'temp__' + self.getName()
        output_filename = f'{output_dir}/{self.getName()}'
        # Get the reader and the writer
        reader = get_readers(p)
        writer = get_writers(p)
        try:
            compression = writer[0]['compression']
            extension = '.laz' if compression == 'laszip' or compression == 'lazperf' else '.las'
        except KeyError:
            # Get the extension for the output
            extension = '.' + writer[0]['type'].split('.')[1] + '.las' if writer[0]['type'].split('.')[1] == 'copc' else '.' + writer[0]['type'].split('.')[1]

        # If there is a buffer
        if self.buffer:
            # If the writer is 'writers.copc' or 'writer.laz' or 'writers.copc'
            if writer[0]['type'].split('.')[1] in ['las', 'laz', 'copc']:
                # Insert the filter to assign the wittheld flag to the buffer
                p.insert(1, self.assign)
        if self.remove_buffer:
            # Remove the buffer
            p.insert(len(p) - 1, bounds.removeBuffer(self.bounds_without_buffer))

        if self.cloud:
            # If there is the ferry filter attribute in the tile instance
            if not self.cloud.classFlags:
                # Insert the filter to add the ClassFlags dimension
                p.insert(1, cloud.addClassFlags())

        # The pipeline must contain a reader AND a writer
        if not reader:
            sys.exit("Please add a reader to your pipeline.")
        elif not writer:
            sys.exit("Please add a writer to your pipeline.")

        # If the input is a single file
        if single_file:
            # Add a crop filter to divide the cloud in small tiles
            p.insert(1, cloud.crop(self.bounds))

        # Add the filename option in the pipeline's reader to get the right file
        reader[0]['filename'] = self.filepath
        # Add the filename option in the pipeline's writer to write the result in the right file
        writer[0]['filename'] = output_filename + extension

        p = pdal.Pipeline(json.dumps(p))

        return p, temp_name

    def split(self, distTileX, distTileY, nTiles=None):
        """Split the tile in small parts of given sizes"""
        current_minx = self.bounds.minx
        current_maxx = current_minx + distTileX if current_minx + distTileX <= self.bounds.maxx else self.bounds.maxx
        current_miny = self.bounds.miny
        current_maxy = current_miny + distTileY if current_miny + distTileY <= self.bounds.maxy else self.bounds.maxy
        # If it's a dry run, 'cpt' will count the number of tiles created
        cpt = 0

        while current_maxx <= self.bounds.maxx and current_maxy <= self.bounds.maxy and (cpt < nTiles if nTiles else True):
            # Create the bounds for the small tile
            b = bounds.Bounds(current_minx, current_miny, current_maxx, current_maxy)
            # Create its name (minx_miny)
            name = str(int(b.minx)) + '_' + str(int(b.miny))
            # Create the tile
            t = Tile(filepath=self.filepath, output_dir=self.output_dir, json_pipeline=self.json_pipeline, name=name, bounds=b, buffer=self.buffer, remove_buffer=self.remove_buffer, cloud_object=self.cloud)
            # Add the width given by the user to shift right to create a new tile
            current_minx += distTileX
            current_maxx += distTileX

            # If the current maxx value exceeds the right edge of the cloud
            if current_maxx >= self.cloud.bounds.maxx:
                # And if there is a piece of cloud non-processed
                if t.bounds.maxx < self.cloud.bounds.maxx:
                    # Calculate the offset distance
                    dist = self.cloud.bounds.maxx - t.bounds.maxx
                    # Shift
                    current_maxx = t.bounds.maxx + dist
                else:
                    # Return to the left edge to create new tiles
                    current_minx = self.bounds.minx
                    current_maxx = current_minx + distTileX
                    # Move down from the height value given by the user
                    current_miny += distTileY
                    current_maxy += distTileY

            # If the current maxy value exceeds the top edge of the cloud
            if current_maxy > self.cloud.bounds.maxy:
                # And if there is a piece of cloud non-processed
                if t.bounds.maxy < self.cloud.bounds.maxy:
                    # Calculate the offset distance
                    dist = self.cloud.bounds.maxy - t.bounds.maxy
                    # Shift
                    current_maxy = t.bounds.maxy + dist

            cpt += 1
            yield t

    def __str__(self):
        if self.bounds:
            return f'{self.bounds} - {self.filepath}'
        else:
            return f'{self.filepath}'



"""
File manager.

Responsible for retrieving all files requested by the user
"""

import os.path
import pickle
import sys
from os import listdir
from os.path import join
import statistics
import subprocess
import json


def getFiles(input_directory, nFiles=None):
    """Returns the files of the input directory"""
    # If it's not a dry run, all the files are returned
    if not nFiles:
        try:
            for f in listdir(input_directory):
                yield join(input_directory, f)
        except NotADirectoryError:
            sys.exit("The input attribute of your configuration file does not designate a directory. Maybe you are "
                     "trying to process a single file ? Check your -it option.")
    # If it's a dry run, only the biggest nFiles of the directory are returned
    else:
        try:
            # Get all the files
            files = [join(input_directory, f) for f in listdir(input_directory)]
            # Create tuples (filepath, filesize)
            filesSize = [(join(input_directory, f), os.path.getsize(f)) for f in files]
            # Sort files in descending order
            filesSize.sort(key=lambda tup: tup[1], reverse=True)
            # Get the first nFiles
            for i in range(nFiles):
                yield filesSize[i][0]
        except NotADirectoryError:
            sys.exit("The input attribute of your configuration file does not designate a directory. Maybe you are "
                     "trying to process a single file ? Check your -it option.")


def getSerializedPipelines(temp_directory):
    """Returns the pipelines that have been serialized"""
    for tmp in listdir(temp_directory):
        # Open the serialized pipeline
        with open(join(temp_directory, tmp), 'rb') as p:
            # Deserialize it
            pipeline = pickle.load(p)
        yield pipeline


def getEmptyWeight(output_directory):
    # Get the output directory files size in bytes
    weights_bytes = [os.path.getsize(join(output_directory, f)) for f in listdir(output_directory)]
    # Convert it in ko
    weights_ko = [round(b / 1024, 2) for b in weights_bytes]
    # Calculate the deciles
    if len(weights_ko) >= 2:
        deciles = [round(q, 2) for q in statistics.quantiles(weights_ko, n=10)]
        # And retrieve files whose weight is in the first decile
        weight_files = [join(output_directory, f) for f in listdir(output_directory) if round(os.path.getsize(join(output_directory, f)) / 1024, 2) <= deciles[0]]
        removeEmptyFiles(weight_files)
    else:
        pass
        #removeEmptyFiles([join(output_directory, f) for f in listdir(output_directory)])


def removeEmptyFiles(files):
    # For each file
    for f in files:
        # Run pdal info
        pdal_info = subprocess.run(['pdal', 'info', f, "--dimensions", "X"],
                                   stderr=subprocess.PIPE,
                                   stdout=subprocess.PIPE)
        info = json.loads(pdal_info.stdout.decode())
        # Get the number of points
        if info['stats']['statistic'][0]['count'] == 0:
            os.remove(f)
